const CaseSensitivePathsPlugin = require('case-sensitive-paths-webpack-plugin');
const WatchMissingNodeModulesPlugin = require('watch-missing-node-modules-webpack-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const colors = require('chalk');
const IS_DEV = process.env.NODE_ENV !== 'production';
const path = require('path'); // eslint-disable-line
const paths = require('../../../config/paths');
const cwd = process.cwd();
const appPath = paths.appPath(cwd);

const baseDir = appPath.appPath;
const srcPath = appPath.appSrc;
const buildPath = appPath.appBuild;
const publicPath = '/';
const publicUrl = '';
const webpack = require(baseDir + '/node_modules/webpack');
const ExtractTextPlugin = require(baseDir + '/node_modules/extract-text-webpack-plugin');


const entry = {
  index: `${srcPath}/index.js`,
};

const config = {
  context: baseDir,
  entry,
  output: {
    path: buildPath,
    filename: '[name].min.js',
    chunkFilename: 'chunks/[name]/[chunkhash].js',
    publicPath: publicPath
  },
  module: {
    noParse: [/moment.js/],
    rules: [
      {
        test: /\.js$/,
        exclude: /node_modules/,
        loader: 'babel-loader',
      }, {
        test: /\.css/,
        loader: ExtractTextPlugin.extract({
          fallback: 'style-loader',
          use: ['css-loader'],
        }),
      },
      {
        test: /\.(mp3|wav|ogg)$/,
        exclude: /node_modules/,
        loader: 'file-loader',
        query: {
          // CSS图片目录
          name: IS_DEV ? '[path][name].[ext]' : 'assets/mp3/[hash].[ext]',
        }
      },
      {
        test: /\.(ai|eps|ttf|svg)$/,
        exclude: /node_modules/,
        loader: 'file-loader',
        query: {
          // 资源目录
          name: IS_DEV ? '[path][name].[ext]' : 'assets/file/[hash].[ext]',
        }
      },
      {
        test: /\.mp4/,
        exclude: /node_modules/,
        loader: 'file-loader',
        query: {
          // CSS图片目录
          name: IS_DEV ? '[path][name].[ext]' : 'assets/video/[hash].[ext]',
        }
      }, {
        test: /\.(png|jpg|gif)$/,
        loader: 'url-loader',
        query: {
          limit: 10000,
          // CSS图片目录
          name: IS_DEV ? '[path][name].[ext]' : 'assets/[name].[ext]',
        }
      }, {
        test: /\.less$/,
        loader: ExtractTextPlugin.extract({
          fallback: 'style-loader',
          use: [
            {
              loader: 'css-loader',
              // https://github.com/webpack-contrib/css-loader
              options: {
                localIdentName: (IS_DEV ? '[name]_[local]_' : '[sx]') + '[hash:base64:5]',
                modules: true,
                camelCase: true,
                // minimize: true, // next组件里的css也需要跑一次压缩去重, 改为 OptimizeCssAssetsPlugin 统一处理
                sourceMap: IS_DEV
              }
            }, {
              loader: 'less-loader',
              options: {
                sourceMap: IS_DEV
              }
            }],
        }),
      },
    ]
  },
  plugins: [
    new webpack.ProvidePlugin({
      React: 'react',
      ReactDom: 'react-dom',
    }),
    new webpack.DefinePlugin({
      __DEV__: JSON.stringify( IS_DEV ),
      'process.env': {
        NODE_ENV: JSON.stringify( IS_DEV ? 'development' : 'production'),
        PUBLIC_URL: JSON.stringify(publicUrl)
      }
    }),
    new HtmlWebpackPlugin({
      inject: true,
      template: appPath.appHtml,
    }),
    // This is necessary to emit hot updates (currently CSS only):
    // new webpack.HotModuleReplacementPlugin(),
    // Watcher doesn't work well if you mistype casing in a path so we use
    // a plugin that prints an error when you attempt to do this.
    // See https://github.com/facebookincubator/create-react-app/issues/240
    new CaseSensitivePathsPlugin(),
    // If you require a missing module and then `npm install` it, you still have
    // to restart the development server for Webpack to discover it. This plugin
    // makes the discovery automatic so you don't have to restart.
    // See https://github.com/facebookincubator/create-react-app/issues/186
    // new WatchMissingNodeModulesPlugin(appPath.appNodeModules),
    new ExtractTextPlugin({filename: '[name].min.css', allChunks: true}),
  ],
  resolve: {
    extensions: ['.js', '.json', '.jsx'],
  }
};

try {
  // 离线支持
  // 本地需要 localhost 访问
  // 线上需要 HTTPS 协议
  // 配置 https://github.com/NekR/offline-plugin/blob/master/docs/options.md
  const offlinePluginConfig = require(baseDir + '/offlinePlugin.config.js');
  if (offlinePluginConfig) {
    const OfflinePlugin = require(baseDir + '/node_modules/offline-plugin');
    config.plugins.push(
      new OfflinePlugin(offlinePluginConfig)
    )
  }
} catch (e) {
  console.log('未发现 offlinePlugin.config.js, 不支持离线模式.');
}

module.exports = config; // eslint-disable-line
